@@ -133,11 +133,12 @@ class Spec122Dataset(Dataset):
         self.add_constant_key("ORIGIN", "National Solar Observatory")
         self.add_constant_key("TELESCOP", "Daniel K. Inouye Solar Telescope")
         self.add_constant_key("OBSERVAT", "Haleakala High Altitude Observatory Site")
+        self.add_constant_key("BUNIT", "ct")
         if instrument:
             self.add_constant_key("INSTRUME", KNOWN_INSTRUMENT_TABLES[instrument])
 
         # WCS
-        self.add_constant_key("CRDATEn")
+        self.add_constant_key("CRDATE<n>")
 
         # Level 0 spec 214 key copying
         if isinstance(file_schema, Level0Spec214Schema):
@@ -184,11 +185,11 @@ class Spec122Dataset(Dataset):
 
     @key_function(
         "WCSAXES",
-        "CRPIXn",
-        "CRVALn",
-        "CDELTn",
-        "CUNITn",
-        "CTYPEn",
+        "CRPIX<n>",
+        "CRVAL<n>",
+        "CDELT<n>",
+        "CUNIT<n>",
+        "CTYPE<n>",
     )
     def wcs_keys(self, key: str):
         return self.fits_wcs.to_header()[key]
@@ -201,7 +202,7 @@ class Spec122Dataset(Dataset):
         wcs.wcs.set()
         return wcs.to_header()[key]
 
-    @key_function("PCi_j")
+    @key_function("PC<i>_<j>")
     def pc_keys(self, key: str):
         i = self.array_ndim - int(key[2])
         j = self.array_ndim - int(key[-1])
