@@ -6,9 +6,11 @@ from dataclasses import dataclass
 
 import astropy.units as u
 import numpy as np
-from astropy.coordinates import GCRS, EarthLocation, SkyCoord
+import numpy.typing as npt
+from astropy.coordinates import GCRS, SkyCoord
 from astropy.coordinates.matrix_utilities import rotation_matrix
 from astropy.time import Time
+from astropy.wcs import WCS
 from dkist_fits_specifications import spec214
 from hashids import Hashids
 from scipy.stats import stats
@@ -18,6 +20,7 @@ from sunpy.util.decorators import cached_property_based_on
 from dkist_data_simulator.dataset import Dataset, Omit, key_function
 from dkist_data_simulator.schemas import Schema, TimeKey
 from dkist_data_simulator.spec122 import KNOWN_INSTRUMENT_TABLES
+from dkist_data_simulator.util import location_of_dkist
 
 __all__ = ["Spec214Dataset", "Spec214Schema"]
 
@@ -34,14 +37,21 @@ class Spec214Schema(Schema):
         dnaxis: int,
         deaxes: int,
         daaxes: int,
+        npropos: int = 1,
+        nexpers: int = 1,
         instrument=None,
         random=None,
     ):
         random = random or np.random.default_rng()
         sections = spec214.load_expanded_spec214(
-            NAXIS=naxis, DNAXIS=dnaxis, DEAXES=deaxes, DAAXES=daaxes
+            NAXIS=naxis,
+            DNAXIS=dnaxis,
+            DEAXES=deaxes,
+            DAAXES=daaxes,
+            NPROPOS=npropos,
+            NEXPERS=nexpers,
         )
-        sections.pop("compression")
+        sections.pop("compression", None)
         if instrument:
             if instrument not in KNOWN_INSTRUMENT_TABLES.keys():
                 raise ValueError(
@@ -51,7 +61,7 @@ class Spec214Schema(Schema):
             for table in KNOWN_INSTRUMENT_TABLES.keys():
                 if table == instrument:
                     continue
-                sections.pop(table)
+                sections.pop(table, None)
 
         super().__init__(self.sections_from_dicts(sections.values(), random=random))
 
@@ -91,6 +101,8 @@ class Spec214Dataset(Dataset):
                 dnaxis=len(dataset_shape),
                 deaxes=len(dataset_shape) - len(data_shape),
                 daaxes=len(data_shape),
+                npropos=1,
+                nexpers=1,
                 instrument=instrument,
             ),
             dataset_shape=dataset_shape,
@@ -154,18 +166,35 @@ class Spec214Dataset(Dataset):
 
         # WCS
         self.add_constant_key("WCSNAME", "Helioprojective-cartesian")
-        self.add_constant_key("CRDATEn")
+        self.add_constant_key("CRDATE<n>")
         self.add_remove_key("LATPOLE")
+        # Which file axis does stokes vary long (Python axis number)
+        # if None then it varies along no axis and is always "I".
+        self.stokes_file_axis = None
 
         # Datacenter
         self.add_constant_key("DSETID")
         self.add_constant_key("RRUNID")
         self.add_constant_key("RECIPEID")
         self.add_constant_key("RINSTID")
-        self.add_constant_key("PROP_ID")
-        self.add_constant_key("EXPER_ID")
-
-        # Dataset
+        self.add_constant_key("PROP_ID", "PROPOSAL")
+        self.add_constant_key("EXPER_ID", "EXPERIMENT")
+        self.add_constant_key("NPROPOS", 1)
+        self.add_constant_key("NEXPERS", 1)
+        self.add_constant_key("PROPID01", "PROPOSAL")
+        self.add_constant_key("EXPRID01", "EXPERIMENT")
+        self.add_constant_key("WKFLNAME", "WORKFLOW")
+        self.add_constant_key("WKFLVERS", "WORKFLOWVERSION")
+        self.add_constant_key("HLSVERS", "HLSVERSION")
+        self.add_constant_key("IDSPARID", 1)
+        self.add_constant_key("IDSOBSID", 2)
+        self.add_constant_key("IDSCALID", 3)
+        self.add_constant_key("OBSPR_ID", "OPID")
+        self.add_constant_key("IP_ID", "IPID")
+        self.add_constant_key("HEADVERS", "HEADERVERSION")
+        self.add_constant_key("HEAD_URL", "HEADERURL")
+        self.add_constant_key("INFO_URL", "INFOURL")
+        self.add_constant_key("CAL_URL", "CALURL")
 
         hashids = Hashids()
         dsetid = hashids.encode(self.random.integers(0, 1000000))
@@ -210,7 +239,7 @@ class Spec214Dataset(Dataset):
         "TELAPSE",
     )
     def date_obs(self, key: str):
-        delta = datetime.timedelta(seconds=self.index * self.time_delta)
+        delta = datetime.timedelta(seconds=self.time_index * self.time_delta)
         frame_start = self.start_time + delta
         frame_end = frame_start + self.exposure_time
 
@@ -231,7 +260,7 @@ class Spec214Dataset(Dataset):
     @property
     def observer(self):
         date_avg = Time(self.date_obs("DATE-AVG"))
-        dkist_pos = EarthLocation.of_site("dkist")
+        dkist_pos = location_of_dkist
         dkist_itrs = dkist_pos.get_itrs(date_avg)
         dkist_hgs = dkist_pos.get_gcrs(date_avg).transform_to(
             HeliographicStonyhurst(obstime=date_avg)
@@ -254,13 +283,13 @@ class Spec214Dataset(Dataset):
         if key == "OBS_VR":
             return dkist_hgs.velocity.d_z.to_value(u.km / u.s)
 
-    @key_function("NAXISn")
+    @key_function("NAXIS<n>")
     def naxis(self, key: str):
         fits_ind = int(key[-1])
         ind = self.array_ndim - fits_ind
         return self.array_shape[ind]
 
-    @key_function("NBINn", "NBIN", "SWBIN1", "HWBIN1", "SWBIN2", "HWBIN2")
+    @key_function("NBIN<n>", "NBIN", "SWBIN1", "HWBIN1", "SWBIN2", "HWBIN2")
     def nbin(self, key: str):
         return 1
 
@@ -291,11 +320,11 @@ class Spec214Dataset(Dataset):
 
     @key_function(
         "WCSAXES",
-        "CRPIXn",
-        "CRVALn",
-        "CDELTn",
-        "CUNITn",
-        "CTYPEn",
+        "CRPIX<n>",
+        "CRVAL<n>",
+        "CDELT<n>",
+        "CUNIT<n>",
+        "CTYPE<n>",
     )
     def wcs_keys(self, key: str):
         return self.fits_wcs.to_header()[key]
@@ -308,13 +337,19 @@ class Spec214Dataset(Dataset):
         wcs.wcs.set()
         return wcs.to_header()[key]
 
-    @key_function("PCi_j")
+    @key_function("PC<i>_<j>")
     def pc_keys(self, key: str):
-        i = self.array_ndim - int(key[2])
-        j = self.array_ndim - int(key[-1])
-        default = self.fits_wcs.wcs.pc[j, i]
+        i = int(key[2]) - 1
+        j = int(key[-1]) - 1
+        default = self.fits_wcs.wcs.pc[i, j]
         return self.fits_wcs.to_header().get(key, default)
 
+    @key_function("STOKES")
+    def stokes(self, key: str):
+        if self.stokes_file_axis is None:
+            return "I"
+        return ["I", "Q", "U", "V"][self.file_index[self.stokes_file_axis]]
+
     ###########################################################################
     # Camera
     ###########################################################################
@@ -329,6 +364,13 @@ class Spec214Dataset(Dataset):
     ###########################################################################
     # Dataset
     ###########################################################################
+    @key_function("FRAMEWAV")
+    def framewav(self, key: str):
+        """
+        Omit framewav unless explicitly added.
+        """
+        return Omit()
+
     def dnaxis(self, key: str):
         fits_ind = int(key[-1])
         ind = self.dataset_ndim - fits_ind
@@ -409,7 +451,7 @@ class Spec214Dataset(Dataset):
         "DATARMS",
         "DATAKURT",
         "DATASKEW",
-        "DATAPpp",
+        "DATAP<pp>",
     )
     def stats(self, key: str):
         if key == "DATAMIN":
@@ -617,3 +659,100 @@ class WCSAMixin:
             return "TOPOCENT"
         if key.startswith("VELOSYS"):
             return 0
+
+
+@dataclass
+class TimeVaryingWCSGenerator:
+    """
+    This class can be used to generate a spatial `~astropy.wcs.WCS` which
+    varies with time.
+
+    The CRVAL and PC matrix vary with time all other parameters are fixed.
+    """
+
+    crval: npt.ArrayLike  # naxis long
+    """Initial value of the reference coordinate."""
+
+    rotation_angle: u.Quantity  # scalar
+    """Initial value of the rotation angle."""
+
+    crpix: npt.ArrayLike  # naxis long
+    """The reference pixel."""
+
+    cdelt: npt.ArrayLike  # naxis long
+    """The plate scale."""
+
+    pointing_shift_rate: u.Quantity  # naxis long, u.deg / u.s
+    """The rate at which the pointing should shift."""
+
+    rotation_shift_rate: u.Quantity  # scalar, u.deg / u.s
+    """The rate at which the rotation matrix should change."""
+
+    cunit: Tuple[str, ...] = ("arcsec", "arcsec")
+    """The units for the WCS axes."""
+
+    ctype: Tuple[str, ...] = ("HPLN-TAN", "HPLT-TAN")
+    """The types of the WCS axes."""
+
+    jitter: bool = False
+    """
+    If `True` the rate of change will randomly change.
+
+    Leading to a non-linear, but monotonic, change.
+    """
+
+    static_axes: Tuple[int, ...] = None
+    """Any axes in the WCS which do not vary."""
+
+    @property
+    def constant_wcs(self):
+        naxis = len(self.crpix)
+        w = WCS(naxis=naxis)
+        w.wcs.crpix = self.crpix
+        w.wcs.cdelt = self.cdelt
+        w.wcs.cunit = self.cunit
+        w.wcs.ctype = self.ctype
+        w.wcs.crval = self.crval
+        w.wcs.pc = np.identity(naxis)
+        return w
+
+    @property
+    def random_jitter(self):
+        if not self.jitter:
+            return 1
+        return np.random.random_sample()
+
+    @property
+    def varying_axes(self):
+        static_axes = self.static_axes or []
+        return np.array([i for i in range(len(self.crpix)) if i not in static_axes])
+
+    def generate_wcs(self, dt: u.s):
+        """
+        Generate the WCS based on the time offset.
+        """
+        w = self.constant_wcs
+
+        pointing_shift_rate = self.random_jitter * self.pointing_shift_rate
+        crval_cunit_zip = [[self.crval[i], self.cunit[i]] for i in self.varying_axes]
+        new_crval = [u.Quantity(cv, unit=cu) for cv, cu in crval_cunit_zip]
+        new_crval = u.Quantity(
+            [cv + (psr * dt) for cv, psr in zip(new_crval, pointing_shift_rate)]
+        )
+
+        rotation_shift_rate = self.random_jitter * self.rotation_shift_rate
+        rotation_angle = self.rotation_angle + rotation_shift_rate * dt
+
+        crval = np.array(w.wcs.crval)
+        crval[self.varying_axes] = new_crval.value
+        w.wcs.crval = crval
+
+        pc = np.array(w.wcs.pc)
+        celestial_pc = rotation_matrix(rotation_angle)[:2, :2]
+        # TODO: Use numpy indexing here not this travesty
+        for x, i in enumerate(self.varying_axes):
+            for y, j in enumerate(self.varying_axes):
+                pc[i, j] = celestial_pc[x, y]
+        w.wcs.pc = pc
+
+        return w
