@@ -6,7 +6,7 @@ from astropy.wcs import WCS
 from sunpy.util.decorators import cached_property_based_on
 
 from dkist_data_simulator.dataset import key_function
-from dkist_data_simulator.spec214.core import Spec214Dataset
+from dkist_data_simulator.spec214.core import Spec214Dataset, TimeVaryingWCSGenerator
 
 
 class BaseVBIDataset(Spec214Dataset):
@@ -22,7 +22,7 @@ class BaseVBIDataset(Spec214Dataset):
         linewave,
         detector_shape=(4096, 4096),
         vbi_camera: Literal["red", "blue"] = "red",
-        mosaic: bool = True
+        mosaic: bool = False
     ):
 
         if vbi_camera.lower() not in ["red", "blue"]:
@@ -76,6 +76,10 @@ class BaseVBIDataset(Spec214Dataset):
     def current_dsps_repeat(self) -> int:
         return self.file_index[0]
 
+    @property
+    def data(self):
+        return np.random.random(self.array_shape)
+
 
 class SimpleVBIDataset(BaseVBIDataset):
     """
@@ -84,9 +88,23 @@ class SimpleVBIDataset(BaseVBIDataset):
 
     name = "vbi-simple"
 
-    @property
-    def data(self):
-        return np.random.random(self.array_shape)
+    def __init__(
+        self,
+        n_time,
+        time_delta,
+        *,
+        linewave,
+        detector_shape=(4096, 4096),
+        vbi_camera: Literal["red", "blue"] = "red"
+    ):
+        super().__init__(
+            n_time,
+            time_delta,
+            linewave=linewave,
+            detector_shape=detector_shape,
+            vbi_camera=vbi_camera,
+            mosaic=False,
+        )
 
     @property
     def fits_wcs(self):
@@ -104,9 +122,20 @@ class SimpleVBIDataset(BaseVBIDataset):
 
 
 class MosaicedVBIBlueDataset(BaseVBIDataset):
-    def __init__(self, n_time, time_delta, *, linewave):
+    """
+    A mosaik'd VBI-BLUE dataset with real WCS values.
+    """
+
+    name = "vbi-mosaic-blue"
+
+    def __init__(self, n_time, time_delta, *, linewave, detector_shape=(4096, 4096)):
         super().__init__(
-            n_time, time_delta, linewave=linewave, vbi_camera="blue", mosaic=True
+            n_time,
+            time_delta,
+            linewave=linewave,
+            vbi_camera="blue",
+            mosaic=True,
+            detector_shape=detector_shape,
         )
 
         self.add_constant_key("VBISTPAT", "5, 6, 3, 2, 1, 4, 7, 9")
@@ -159,7 +188,7 @@ class MosaicedVBIBlueDataset(BaseVBIDataset):
     def moasic_keys(self, key: str):
         axis_length = 3
         if key == "VBISTP":
-            return self.current_camera_pos
+            return int(self.current_camera_pos)
         if key == "MINDEX1":
             return int((self.current_camera_pos - 1) % axis_length + 1)
         if key == "MINDEX2":
@@ -171,3 +200,37 @@ class MosaicedVBIBlueDataset(BaseVBIDataset):
             "MAXIS2": 3,
         }
         return constant_keys.get(key, super().moasic_keys(key))
+
+
+class TimeDependentVBIDataset(SimpleVBIDataset):
+    """
+    A version of the VBI dataset where the CRVAL and PC matrix change with time.
+    """
+
+    name = "vbi-time-dependent"
+
+    def __init__(
+        self,
+        n_time,
+        time_delta,
+        *,
+        linewave,
+        pointing_shift_rate=10 * u.arcsec / u.s,
+        rotation_shift_rate=0.5 * u.deg / u.s,
+        **kwargs
+    ):
+        super().__init__(n_time, time_delta, linewave=linewave, **kwargs)
+
+        self.wcs_generator = TimeVaryingWCSGenerator(
+            crval=(0, 0),
+            rotation_angle=-2 * u.deg,
+            crpix=(self.array_shape[1] / 2, self.array_shape[0] / 2),
+            cdelt=[self.plate_scale.to_value(u.arcsec / u.pix) for i in range(2)],
+            pointing_shift_rate=u.Quantity([pointing_shift_rate, pointing_shift_rate]),
+            rotation_shift_rate=rotation_shift_rate,
+            jitter=False,
+        )
+
+    @property
+    def fits_wcs(self):
+        return self.wcs_generator.generate_wcs(self.time_index * self.time_delta * u.s)
