@@ -2,7 +2,8 @@ import astropy.units as u
 import numpy as np
 from astropy.wcs import WCS
 
-from .core import Spec214Dataset
+from dkist_data_simulator.dataset import key_function
+from dkist_data_simulator.spec214.core import Spec214Dataset
 
 
 class BaseVTFDataset(Spec214Dataset):
@@ -29,7 +30,7 @@ class BaseVTFDataset(Spec214Dataset):
         array_shape = list(detector_shape)
 
         dataset_shape_rev = list(detector_shape) + [n_wave, n_repeats]
-        if n_stokes:
+        if n_stokes > 1:
             dataset_shape_rev += [n_stokes]
 
         super().__init__(
@@ -59,16 +60,33 @@ class BaseVTFDataset(Spec214Dataset):
         self.add_constant_key("DWNAME4", "time")
         self.add_constant_key("DUNIT4", "s")
 
-        if n_stokes:
+        if n_stokes > 1:
             self.add_constant_key("DTYPE5", "STOKES")
             self.add_constant_key("DPNAME5", "stokes")
             self.add_constant_key("DWNAME5", "stokes")
             self.add_constant_key("DUNIT5", "")
+            self.stokes_file_axis = 0
 
+        self.linewave = linewave
         self.add_constant_key("LINEWAV", linewave.to_value(u.nm))
 
         # TODO: Check this value is right
         self.plate_scale = 0.012 * u.arcsec / u.pix
+        self.n_stokes = n_stokes
+
+    @key_function("FRAMEWAV")
+    def framewav(self, key: str):
+        """
+        Add a random framewav around the line centre
+        """
+        return self.linewave.to_value(u.nm) - (np.random.random() - 0.5) * 10
+
+    @property
+    def non_temporal_file_axes(self):
+        if self.n_stokes > 1:
+            # This is the index in file shape so third file dimension
+            return (0,)
+        return super().non_temporal_file_axes
 
 
 class SimpleVTFDataset(BaseVTFDataset):
