@@ -1,5 +1,5 @@
 """
-This file contains tests which verify that inventory generation works.
+this file contains tests which verify that inventory generation works.
 
 Because this file leads to a circular test-time dependency between
 dkist-inventory and the simulator, these tests are in their own file which is
@@ -18,12 +18,14 @@ dkist_inventory = pytest.importorskip("dkist_inventory", minversion="0.8.0")
 
 import astropy.units as u
 import numpy as np
+from astropy.table import Table
 from astropy.wcs import WCS
-from dkist_header_validator import spec214_validator
-from dkist_inventory.inventory import extract_inventory, sort_headers
+from dkist_inventory.inventory import extract_inventory
+from dkist_inventory.header_parsing import HeaderParser
 
+from dkist_data_simulator.dataset import key_function
 from dkist_data_simulator.spec214 import Spec214Dataset
-from dkist_data_simulator.spec214.cryo import SimpleCryoDataset
+from dkist_data_simulator.spec214.cryo import SimpleCryoCIDataset, SimpleCryoSPDataset
 from dkist_data_simulator.spec214.dlnirsp import SimpleDLNIRSPDataset
 from dkist_data_simulator.spec214.vbi import MosaicedVBIBlueDataset, SimpleVBIDataset
 from dkist_data_simulator.spec214.visp import SimpleVISPDataset
@@ -46,12 +48,19 @@ class DatasetTest214(Spec214Dataset):
     def data(self):
         return np.random.random(self.array_shape)
 
+    @key_function("FRAMEWAV")
+    def framewav(self, key: str):
+        """
+        Add a random framewav around the line centre
+        """
+        return (np.random.random() - 0.5) * 100
+
 
 @pytest.mark.parametrize(
     "ds",
     (
         DatasetTest214(
-            dataset_shape=(2, 2, 2, 40, 50), array_shape=(40, 50), time_delta=10
+            dataset_shape=(2, 2, 40, 50), array_shape=(40, 50), time_delta=10
         ),
         SimpleVISPDataset(
             n_maps=2, n_steps=3, n_stokes=4, time_delta=10, linewave=500 * u.m
@@ -66,11 +75,17 @@ class DatasetTest214(Spec214Dataset):
         SimpleVTFDataset(
             n_wave=2, n_repeats=3, n_stokes=0, time_delta=10, linewave=400 * u.nm
         ),
-        SimpleCryoDataset(
-            n_raster=2, n_scans=3, n_stokes=4, time_delta=10, linewave=500 * u.m
+        SimpleCryoSPDataset(
+            n_maps=3, n_steps=3, n_stokes=4, time_delta=10, linewave=500 * u.m
         ),
-        SimpleCryoDataset(
-            n_raster=2, n_scans=3, n_stokes=0, time_delta=10, linewave=500 * u.m
+        SimpleCryoSPDataset(
+            n_maps=3, n_steps=3, n_stokes=0, time_delta=10, linewave=500 * u.m
+        ),
+        SimpleCryoCIDataset(
+            n_maps=3, n_steps=3, n_stokes=4, time_delta=10, linewave=500 * u.m
+        ),
+        SimpleCryoCIDataset(
+            n_maps=3, n_steps=3, n_stokes=0, time_delta=10, linewave=500 * u.m
         ),
         SimpleDLNIRSPDataset(
             n_exposures=3, n_stokes=4, time_delta=10, linewave=400 * u.nm
@@ -87,8 +102,10 @@ class DatasetTest214(Spec214Dataset):
         "SimpleVBIDataset",
         "SimpleVTFDataset-polarized",
         "SimpleVTFDataset",
-        "SimpleCRYODataset-polarized",
-        "SimpleCRYODataset",
+        "SimpleCRYOSPDataset-polarized",
+        "SimpleCRYOSPDataset",
+        "SimpleCRYOCIDataset-polarized",
+        "SimpleCRYOCIDataset",
         "SimpleDLNIRSPDataset-polarized",
         "SimpleDLNIRSPDataset",
         "MosaicedVBIBlueDataset",
@@ -97,14 +114,10 @@ class DatasetTest214(Spec214Dataset):
 def test_generate_214(ds):
     headers = ds.generate_headers()
 
-    for h in headers:
-        spec214_validator.validate(h)
-
     # Assert that the datasets generated here pass through gwcs generation and inventory creation.
     # This is the most minimal sanity check possible.
 
     filenames = [f"{i}.fits" for i in range(len(headers))]
-    table_headers, _, sorted_headers = sort_headers(headers, filenames)
-
-    inv = extract_inventory(table_headers)
+    header_parser = HeaderParser.from_headers(headers, filenames=filenames)
+    inv = extract_inventory(header_parser)
     assert isinstance(inv, dict)
